// Copyright 2020 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by client-gen. DO NOT EDIT.

package v1alpha1

import (
	"context"
	"time"

	v1alpha1 "github.com/google/kf/v2/pkg/apis/kf/v1alpha1"
	scheme "github.com/google/kf/v2/pkg/client/kf/clientset/versioned/scheme"
	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	rest "k8s.io/client-go/rest"
)

// SourcePackagesGetter has a method to return a SourcePackageInterface.
// A group's client should implement this interface.
type SourcePackagesGetter interface {
	SourcePackages(namespace string) SourcePackageInterface
}

// SourcePackageInterface has methods to work with SourcePackage resources.
type SourcePackageInterface interface {
	Create(ctx context.Context, sourcePackage *v1alpha1.SourcePackage, opts v1.CreateOptions) (*v1alpha1.SourcePackage, error)
	Update(ctx context.Context, sourcePackage *v1alpha1.SourcePackage, opts v1.UpdateOptions) (*v1alpha1.SourcePackage, error)
	UpdateStatus(ctx context.Context, sourcePackage *v1alpha1.SourcePackage, opts v1.UpdateOptions) (*v1alpha1.SourcePackage, error)
	Delete(ctx context.Context, name string, opts v1.DeleteOptions) error
	DeleteCollection(ctx context.Context, opts v1.DeleteOptions, listOpts v1.ListOptions) error
	Get(ctx context.Context, name string, opts v1.GetOptions) (*v1alpha1.SourcePackage, error)
	List(ctx context.Context, opts v1.ListOptions) (*v1alpha1.SourcePackageList, error)
	Watch(ctx context.Context, opts v1.ListOptions) (watch.Interface, error)
	Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts v1.PatchOptions, subresources ...string) (result *v1alpha1.SourcePackage, err error)
	SourcePackageExpansion
}

// sourcePackages implements SourcePackageInterface
type sourcePackages struct {
	client rest.Interface
	ns     string
}

// newSourcePackages returns a SourcePackages
func newSourcePackages(c *KfV1alpha1Client, namespace string) *sourcePackages {
	return &sourcePackages{
		client: c.RESTClient(),
		ns:     namespace,
	}
}

// Get takes name of the sourcePackage, and returns the corresponding sourcePackage object, and an error if there is any.
func (c *sourcePackages) Get(ctx context.Context, name string, options v1.GetOptions) (result *v1alpha1.SourcePackage, err error) {
	result = &v1alpha1.SourcePackage{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("sourcepackages").
		Name(name).
		VersionedParams(&options, scheme.ParameterCodec).
		Do(ctx).
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of SourcePackages that match those selectors.
func (c *sourcePackages) List(ctx context.Context, opts v1.ListOptions) (result *v1alpha1.SourcePackageList, err error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	result = &v1alpha1.SourcePackageList{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("sourcepackages").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Do(ctx).
		Into(result)
	return
}

// Watch returns a watch.Interface that watches the requested sourcePackages.
func (c *sourcePackages) Watch(ctx context.Context, opts v1.ListOptions) (watch.Interface, error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	opts.Watch = true
	return c.client.Get().
		Namespace(c.ns).
		Resource("sourcepackages").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Watch(ctx)
}

// Create takes the representation of a sourcePackage and creates it.  Returns the server's representation of the sourcePackage, and an error, if there is any.
func (c *sourcePackages) Create(ctx context.Context, sourcePackage *v1alpha1.SourcePackage, opts v1.CreateOptions) (result *v1alpha1.SourcePackage, err error) {
	result = &v1alpha1.SourcePackage{}
	err = c.client.Post().
		Namespace(c.ns).
		Resource("sourcepackages").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(sourcePackage).
		Do(ctx).
		Into(result)
	return
}

// Update takes the representation of a sourcePackage and updates it. Returns the server's representation of the sourcePackage, and an error, if there is any.
func (c *sourcePackages) Update(ctx context.Context, sourcePackage *v1alpha1.SourcePackage, opts v1.UpdateOptions) (result *v1alpha1.SourcePackage, err error) {
	result = &v1alpha1.SourcePackage{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("sourcepackages").
		Name(sourcePackage.Name).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(sourcePackage).
		Do(ctx).
		Into(result)
	return
}

// UpdateStatus was generated because the type contains a Status member.
// Add a +genclient:noStatus comment above the type to avoid generating UpdateStatus().
func (c *sourcePackages) UpdateStatus(ctx context.Context, sourcePackage *v1alpha1.SourcePackage, opts v1.UpdateOptions) (result *v1alpha1.SourcePackage, err error) {
	result = &v1alpha1.SourcePackage{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("sourcepackages").
		Name(sourcePackage.Name).
		SubResource("status").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(sourcePackage).
		Do(ctx).
		Into(result)
	return
}

// Delete takes name of the sourcePackage and deletes it. Returns an error if one occurs.
func (c *sourcePackages) Delete(ctx context.Context, name string, opts v1.DeleteOptions) error {
	return c.client.Delete().
		Namespace(c.ns).
		Resource("sourcepackages").
		Name(name).
		Body(&opts).
		Do(ctx).
		Error()
}

// DeleteCollection deletes a collection of objects.
func (c *sourcePackages) DeleteCollection(ctx context.Context, opts v1.DeleteOptions, listOpts v1.ListOptions) error {
	var timeout time.Duration
	if listOpts.TimeoutSeconds != nil {
		timeout = time.Duration(*listOpts.TimeoutSeconds) * time.Second
	}
	return c.client.Delete().
		Namespace(c.ns).
		Resource("sourcepackages").
		VersionedParams(&listOpts, scheme.ParameterCodec).
		Timeout(timeout).
		Body(&opts).
		Do(ctx).
		Error()
}

// Patch applies the patch and returns the patched sourcePackage.
func (c *sourcePackages) Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts v1.PatchOptions, subresources ...string) (result *v1alpha1.SourcePackage, err error) {
	result = &v1alpha1.SourcePackage{}
	err = c.client.Patch(pt).
		Namespace(c.ns).
		Resource("sourcepackages").
		Name(name).
		SubResource(subresources...).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(data).
		Do(ctx).
		Into(result)
	return
}
